@@ -9,8 +9,10 @@ def get_package_data():
         os.path.join('data/querydata', '*.xml'),
         os.path.join('data/querydata', '*.xml'),
         os.path.join('data/tap', '*.xml'),
+        os.path.join('data/tap', '*.htm'),
         os.path.join('data/scs', '*.xml'),
         os.path.join('data/sia', '*.xml'),
+        os.path.join('data/sia2', '*.xml'),
         os.path.join('data/sla', '*.xml'),
         os.path.join('data/ssa', '*.xml'),
         os.path.join('data/datalink', '*.xml'),
