@@ -7,7 +7,8 @@ from functools import partial
 from urllib.parse import parse_qsl
 
 from pyvo.dal.adhoc import DatalinkResults
-from pyvo.dal.params import find_param_by_keyword, get_converter
+from pyvo.dal.params import find_param_by_keyword, get_converter,\
+    AbstractDalQueryParam, IntervalQueryParam
 from pyvo.dal.exceptions import DALServiceError
 
 import pytest
@@ -186,3 +187,38 @@ def test_inf():
     proc = datalink[0]
 
     proc.process(band=(6000, +np.inf) * u.Angstrom)
+
+
+def test_dal_query_param():
+    class Test(AbstractDalQueryParam):
+        def get_dal_format(self, item):
+            return str(item)
+    # check test_obs behaves like a set but also holds the dal representation
+    test_obs = Test()
+    test_obs.add(1)
+    assert 1 in test_obs
+    assert test_obs.dal == ['1']
+    test_obs.add(2)
+    test_obs.add(3)
+    assert len(test_obs) == 3
+    assert test_obs.dal == ['1', '2', '3']
+    assert {2, 3} < test_obs
+    assert {1, 2, 3, 4} > test_obs
+    test_obs.clear()
+    assert len(test_obs) == 0
+    assert len(test_obs.dal) == 0
+
+
+def test_dal_format():
+    iqp = IntervalQueryParam(unit=u.m, equivalencies=u.spectral())
+    assert '1.0 1.0' == iqp.get_dal_format(1)
+    assert '1.0 2.0' == iqp.get_dal_format((1, 2))
+    assert '1.0 2.0' == iqp.get_dal_format((100*u.cm, 200*u.cm))
+    assert '1.0 2.0' == iqp.get_dal_format((100, 200)*u.cm)
+    assert '0.14989622900000002 1.0' == iqp.get_dal_format((100*u.cm, 2*u.GHz))
+    assert '14.9896229 29.9792458' == iqp.get_dal_format((0.01, 0.02)*u.GHz)
+    # Quantity intervals are corrected in terms of min and max ..
+    assert '1.0 2.0' == iqp.get_dal_format((2, 1)*u.m)
+    # But unitless intervals are not
+    with pytest.raises(ValueError):
+        iqp.get_dal_format((2, 1))
